<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Jürgen Reinhold <jreinhold@freenet.de>
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/


/**
 *
 *
 * @package du_ranking
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License, version 3 or later
 *
 */
class Tx_DuRanking_Controller_TournamentController extends Tx_Extbase_MVC_Controller_ActionController {

	/**
	 * tournamentRepository
	 *
	 * @var Tx_DuRanking_Domain_Repository_TournamentRepository
	 */
	protected $tournamentRepository;
	
	/**
	 * seasonRepository
	 *
	 * @var Tx_DuRanking_Domain_Repository_SeasonRepository
	 */
	protected $seasonRepository;
	
	/**
	 * locationRepository
	 *
	 * @var Tx_DuRanking_Domain_Repository_LocationRepository
	 */
	protected $locationRepository;
	
	/**
	 * personRepository
	 *
	 * @var Tx_DuRanking_Domain_Repository_PersonRepository
	 */
	protected $personRepository;

	/**
	 * Initializes the current action
	 *
	 * @return void
	 */
	public function initializeAction() {
		$this->tournamentRepository = t3lib_div::makeInstance('Tx_DuRanking_Domain_Repository_TournamentRepository');
		$this->seasonRepository = t3lib_div::makeInstance('Tx_DuRanking_Domain_Repository_SeasonRepository');
		$this->locationRepository = t3lib_div::makeInstance('Tx_DuRanking_Domain_Repository_LocationRepository');
		$this->personRepository = t3lib_div::makeInstance('Tx_DuRanking_Domain_Repository_PersonRepository');
		
	}
	
	/**
	 * action show
	 *
	 * @param Tx_DuRanking_Domain_Model_Tournament $tournament
	 * @return void
	 */
	public function showAction(Tx_DuRanking_Domain_Model_Tournament $tournament) {
		
		$this->view->assign('tournament', $tournament);
		$positions = $tournament->getPositions();
		
		$teams = $tournament->getLocation()->getTeams();
		$bonuspoints = $tournament->getSeason()->getBonuspoints();

		// Punkte fr dieses Turnier ermitteln
		$posCount = $positions->count();
		foreach($positions as $pos) {
			$colName = 'number'.$pos->getNumber();
			$points = $this->tournamentRepository->getPointsFromTups($colName, $posCount);
			// Punkte nur einmal berechnen...!
			if ($points === NULL OR $points == "") {
				/*Bonuspoints ? */
				if ($bonuspoints !== NULL AND $bonuspoints > 0) {
					if ($teams !== NULL AND $teams !== "") {
	
						$search_team = $pos->getParticipant()->getTeam();
	
						if ($search_team !== NULL AND $search_team !== "") {
							if (strstr($teams, $search_team) === false) {
								$points = $points + $bonuspoints;
							}
						}
						else {
							$points = $points + $bonuspoints;	/*TN ohne Team*/
						}
					}
				}
				$pos->setPoints($points);
			}
		}
		
		$this->view->assign('positions', $positions);
	}

	/**
	 * action list
	 *
	 * @param Tx_DuRanking_Domain_Model_Demand $demand A demand taken as a basis for filtering the tournaments
	 * @return void
	 */
	public function listAction(Tx_DuRanking_Domain_Model_Demand $demand = NULL) {
		
		$this->view->assign('demand', $demand);
		
		/* options for select */
		$seasons = $this->seasonRepository->findAll();
		$this->view->assign('seasons',$seasons);
		
		$this->view->assign('locations', $this->locationRepository->findAll());
		
		$month = array_merge(array(0 => 'Komplette Saison'), 
							 array(1 => 'Januar'),
							 array(2 => 'Februar'),
							 array(3 => 'März'),
							 array(4 => 'April'),
							 array(5 => 'Mai'),
							 array(6 => 'Juni'),
							 array(7 => 'Juli'),
							 array(8 => 'August'),
							 array(9 => 'September'),
							 array(10 => 'Oktober'),
							 array(11 => 'November'),
							 array(12 => 'Dezember')
							     
		);
		$this->view->assign('all_month', $month);
		
		$this->view->assign('participants',$this->personRepository->findAll());
		
		/* assign result of demand */
		if ($demand !== NULL) {
			$tournaments = $this->tournamentRepository->findDemanded($demand);
			//$this->flashMessages->add('Turnierliste gefiltert!');
		}
		else {
			//Get limit from TypoScript settings or use standard 100
			$limit = (int)$this->settings['tournamentLimit'];
			if(is_int($limit)) {
				$tournaments = $this->tournamentRepository->findLimited($limit);
			}
			else {
				$tournaments = $this->tournamentRepository->findLimited(100);
				//$this->flashMessages->add('TS-setting tournamentLimit: '.$limit);
			}
		}
		$this->view->assign('tournaments',$tournaments);
	}

	/**
	 * action ranking
	 *
	 * @param Tx_DuRanking_Domain_Model_Demand $demand A demand taken as a basis for filtering the tournaments
	 * @return void
	 */
	public function rankingAction(Tx_DuRanking_Domain_Model_Demand $demand = NULL) {
		
		$this->view->assign('demand', $demand);
		
		/* options for select */
		$seasons = $this->seasonRepository->findAll();
		$this->view->assign('seasons',$seasons);
		
		$gender = array_merge(array(0 => 'Gesamt'), 
							  array(1 => 'Damen'),
							  array(2 => 'Herren')
		);
		$this->view->assign('gender', $gender);
		
		$sorting = array_merge(array(0 => 'Punkte'), 
							   array(1 => 'Schnitt')
		);
		$this->view->assign('sorting', $sorting);
		
		/* assign result of demand */
		if ($demand !== NULL) {
			$tournaments = $this->tournamentRepository->findForSeason($demand->getSeason());
			$ranking = new Tx_DuRanking_Domain_Model_Ranking($tournaments->toArray());
			
			if ($demand->getSex() == 2) {
				$rankingPositions = $ranking->getRankingPositionsMale($demand->getSorting());
			}
			elseif ($demand->getSex() == 1) {
				$rankingPositions = $ranking->getRankingPositionsFemale($demand->getSorting());
			}
			else {
				$rankingPositions = $ranking->getRankingPositions($demand->getSorting());
			}
			$this->view->assign('seasoninfo',$demand->getSeason());
			//$this->flashMessages->add('Rangliste!');
			
		}
		/* or assign standard season */
		else {
			$tournaments = $this->tournamentRepository->findForSeason($seasons->getFirst());
			$ranking = new Tx_DuRanking_Domain_Model_Ranking($tournaments->toArray(), 0);
			$rankingPositions = $ranking->getRankingPositions(0);
			
			$this->view->assign('seasoninfo',$seasons->getFirst());
			//$this->flashMessages->add('Bitte eine Rangliste auswhlen!');
		}
		$this->view->assign('tournaments',$tournaments);
		$this->view->assign('rankingPositions',$rankingPositions);
	}
	
	/**
	 * action statistic
	 *
	 * @param Tx_DuRanking_Domain_Model_Demand $demand The season (property of demand object) for filtering the tournaments
	 * @return void
	 */
	public function statisticAction(Tx_DuRanking_Domain_Model_Demand $demand = NULL) {
		
		$this->view->assign('demand', $demand);
		
		/* options for select */
		$seasons = $this->seasonRepository->findAll();
		$this->view->assign('seasons',$seasons);
		
		//Anzahl Turnier in der Saison
		$tournaments = array();
		$statTournaments = array();	//Saison - Anzahl Turniere - Anzahl TN - Gesamtpunktzahl -- Anzahl Spieler - Anz. meiste Turniere - Anz. niedrigste Turniere - Schnitt
		$lfdnr = 1;
		$countParticipant = 0;
		$countPoints = 0;
		foreach ($seasons as $tempSeason) {
			//Anzahl Turniere pro Saison
			$count = $this->tournamentRepository->countTournaments($tempSeason);
			$statTournaments['season'] = $tempSeason->getName();
			$statTournaments['countTournaments'] = $count;
			//Anzahl Teilnehmer pro Saison
			$allTournaments = $this->tournamentRepository->findForSeason($tempSeason);
			foreach ($allTournaments as $tour) {
				$positions = $tour->getPositions();
				foreach($positions as $position) {
					$countParticipant = $countParticipant + 1;
					$countPoints = $countPoints + $position->getPoints();
				}
			}
			//Gesamtpunktzahl aller Teilnehmer bei allen Turnieren pro Saison
			$statTournaments['countParticipant'] = $countParticipant;
			$statTournaments['countPoints'] = $countPoints;
			$tournaments[$lfdnr] = $statTournaments;
			$lfdnr = $lfdnr + 1;
			$countParticipant = 0;
			$countPoints = 0;
		}
		$this->view->assign('tournaments',$tournaments);

		//Top 10 Platzierungen
		if($demand !== NULL) {
			$season = $demand->getSeason();
		}
		else {
			$season = $seasons->getFirst();
		}
		$tournaments = $this->tournamentRepository->findForSeason($season);
		$ranking = new Tx_DuRanking_Domain_Model_Ranking($tournaments->toArray(), 0);
		$top10Positions = $ranking->getTop10RankingPositions();	//Top10 der Rangliste
		//$top10Positions = Teilnehmer - ges.Anz. Turniere - Platz
		$lfdnr = 1;
		$number1 = 0;
		$number2 = 0;
		$number3 = 0;
		$number4 = 0;
		$top = array();		//Array mit allen Daten zu einem Teilnehmer
		$top10 = array();	//Array mit allen $top Arrays, den den TOP10
		foreach($top10Positions as $topRP) {
			//Lfd. Nr., Name und Gesamtanzahl Turniere des Spielers einem Array "$top10" hinzufgen
			$top['lfdnr'] = $lfdnr;
			$top['name'] = $topRP->getParticipant();
			$top['countAll'] = $topRP->getTournamentCount();
			//Alle Turniere eines Top10 Spielers holen
			$tournaments = $this->tournamentRepository->findByPerson($topRP->getParticipant(), $season);
			//In den Turnieren alle 1.Pltze zhlen und dem Array "$top" hinzufgen 
			foreach($tournaments as $tournament) {
				$positions = $tournament->getPositions();
				foreach($positions as $position) {
					//In den Turnieren alle 1.Pltze zhlen und dem Array "$top" hinzufgen
					if($position->getParticipant() == $topRP->getParticipant()) {
						if($position->getNumber()== 1) {
							$number1 = $number1 + 1;
						}//In den Turnieren alle 2.Pltze zhlen und dem Array "$top" hinzufgen
						elseif ($position->getNumber()== 2) {
							$number2 = $number2 + 1;
						}//In den Turnieren alle 3.Pltze zhlen und dem Array "$top" hinzufgen
						elseif ($position->getNumber()== 3) {
							$number3 = $number3 + 1;
						}
						elseif ($position->getNumber()== 4) {
							$number4 = $number4 + 1;
						}
						if($position->getNumber() >= $worst) {
							$worst = $position->getNumber();
						}
					}
				}
			}
			$top['number1'] = $number1;
			$top['number2'] = $number2;
			$top['number3'] = $number3;
			$top['number4'] = $number4;
			$top['worst'] = $worst;
			//$top = LfdNr - Name - Gesamt Anz. Turniere - Anz. Platz 1 - Anz. Platz 2 - Anz. Platz 3 - Anz. Platz 4 - schlechtester Platz
			//$top10 = Top 1 bis Top 10
			$top10[$lfdnr] = $top;
			$lfdnr = $lfdnr + 1;
			//Anzahl zurcksetzen
			$number1 = 0;
			$number2 = 0;
			$number3 = 0;
			$number4 = 0;
			$worst = 0;
		}
		$this->view->assign('top10', $top10);
	}
}
?>