<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011 Jürgen Reinhold <jreinhold@freenet.de>
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/


/**
 *
 *
 * @package du_ranking
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License, version 3 or later
 *
 */
class Tx_DuRanking_Domain_Model_Ranking extends Tx_Extbase_DomainObject_AbstractEntity {
	
	/**
	 * the tournaments for the rankinglist
	 *
	 * @var array
	 */
	protected $tournaments;

	/**
	 * the complete ranking for the tournaments
	 * Tx_Extbase_Persistence_ObjectStorage<Tx_DuRanking_Domain_Model_RankingPosition>
	 *
	 * @var array 
	 */
	protected $rankingPositions;
	
	/**
	 * the male ranking for the tournaments
	 * Tx_Extbase_Persistence_ObjectStorage<Tx_DuRanking_Domain_Model_RankingPosition>
	 *
	 * @var array 
	 */
	protected $rankingPositionsMale;
	
	/**
	 * the female ranking for the tournaments
	 * Tx_Extbase_Persistence_ObjectStorage<Tx_DuRanking_Domain_Model_RankingPosition>
	 *
	 * @var array 
	 */
	protected $rankingPositionsFemale;
	
	/**
	 * the season for the rankinglist
	 *
	 * @var Tx_DuRanking_Domain_Model_Season
	 */
	protected $season;
	
	
	
	/**
	 * __construct
	 *
	 * @param array $tournaments
	 * @return void
	 */
	public function __construct(array $tournaments) {
		
		$this->tournaments = $tournaments;
		$this->rankingPositions = array();
		
		//$this->rankingPositionsFemale = array();
		
		//fge von jedem Turnier den TN-Namen und die Punkte (aus Platzierungen) zu rankingPositions hinzu und setze TurnierAnzahl auf 1
		//ist der Name in dem array schon vorhanden addiere die Punkte dazu und erhhe die Anzahl der Turniere um 1
		foreach($this->tournaments as $tournament) {
			$positions = $tournament->getPositions();
			foreach($positions as $position) {
				$this->addRankingPosition($position->getParticipant(), $position->getPoints());
			}
			$this->season = $tournament->getSeason();
		}
		
		//wenn alle Turniere hinzugefgt wurden
		//durchlaufe rankingPositions und berechne Schnitt und prfe ob TN qualifiziert ist(Abfrage Saison nach AnzahlTurnier und AnzahlSpielsttten)
		$this->calculateRankingPosition($this->season);

	}


	/**
	 * Returns the RankingPositions
	 * Tx_Extbase_Persistence_ObjectStorage<Tx_DuRanking_Domain_Model_RankingPosition>
	 *
	 * @param int $sorting	0 => points, 1 => average
	 * @return array $rankingPositions
	 */
	public function getRankingPositions($sorting) {
		//sortieren der Liste nach $sorting
		$this->sortRankingPosition($sorting);
		
		return $this->rankingPositions;
	}
	
	/**
	 * Returns only  the male RankingPositions
	 * 
	 * @param int $sorting	0 => points, 1 => average
	 * @return array $rankingPositionsMale
	 */
	public function getRankingPositionsMale($sorting) {
		$this->rankingPositionsMale = array();
		
		foreach($this->tournaments as $tournament) {
			$positions = $tournament->getPositions();
			foreach($positions as $position) {
				if($position->getParticipant()->getSex() == 0) {
					$this->addRankingPositionMale($position->getParticipant(), $position->getPoints());
				}
			}
			$this->season = $tournament->getSeason();
		}
		$this->calculateRankingPositionMale($this->season);
		$this->sortRankingPositionMale($sorting);
		
		return $this->rankingPositionsMale;
	}
	
	/**
	 * Returns only the female RankingPositions
	 * 
	 * @param int $sorting	0 => points, 1 => average
	 * @return array $rankingPositionsFemale
	 */
	public function getRankingPositionsFemale($sorting) {
		$this->rankingPositionsFemale = array();
		
		foreach($this->tournaments as $tournament) {
			$positions = $tournament->getPositions();
			foreach($positions as $position) {
				if($position->getParticipant()->getSex() > 0) {
					$this->addRankingPositionFemale($position->getParticipant(), $position->getPoints());
				}
			}
			$this->season = $tournament->getSeason();
		}
		$this->calculateRankingPositionFemale($this->season);
		$this->sortRankingPositionFemale($sorting);
		
		return $this->rankingPositionsFemale;
	}
	
	/**
	 * Returns the Top10 RankingPositions
	 *
	 * @return array $rankingPositions
	 */
	public function getTop10RankingPositions() {
		//sortieren der Liste nach 0 => points
		$this->sortRankingPosition(0);
		
		return array_slice($this->rankingPositions, 0, 10);
	}
	
	   
	/**
	 * Adds a RankingPosition
	 *
	 * @param Tx_DuRanking_Domain_Model_Person $participant
	 * @param int $points
	 * @return void
	 */
	protected  function addRankingPosition($participant, $points) {
		
		if(count($this->rankingPositions) > 0) {
			foreach($this->rankingPositions as $oldPos) {
				if ($oldPos->getParticipant() == $participant) {
					$oldPoints = $oldPos->getPoints();
					$oldPos->setPoints($oldPoints + $points);
					$oldCount = $oldPos->getTournamentCount();
					$oldPos->setTournamentCount($oldCount + 1);
					return;
				}
			}
		}
		$newPos = new Tx_DuRanking_Domain_Model_RankingPosition($participant, $points);
		$this->rankingPositions[] = $newPos;
	}
	
	/**
	 * Adds a RankingPositionMale
	 *
	 * @param Tx_DuRanking_Domain_Model_Person $participant
	 * @param int $points
	 * @return void
	 */
	protected  function addRankingPositionMale($participant, $points) {
		
		if(count($this->rankingPositionsMale) > 0) {
			foreach($this->rankingPositionsMale as $oldPos) {
				if ($oldPos->getParticipant() == $participant) {
					$oldPoints = $oldPos->getPoints();
					$oldPos->setPoints($oldPoints + $points);
					$oldCount = $oldPos->getTournamentCount();
					$oldPos->setTournamentCount($oldCount + 1);
					return;
				}
			}
		}
		$newPos = new Tx_DuRanking_Domain_Model_RankingPosition($participant, $points);
		$this->rankingPositionsMale[] = $newPos;
	}
	
	/**
	 * Adds a RankingPositionFemale
	 *
	 * @param Tx_DuRanking_Domain_Model_Person $participant
	 * @param int $points
	 * @return void
	 */
	protected  function addRankingPositionFemale($participant, $points) {
		
		if(count($this->rankingPositionsFemale) > 0) {
			foreach($this->rankingPositionsFemale as $oldPos) {
				if ($oldPos->getParticipant() == $participant) {
					$oldPoints = $oldPos->getPoints();
					$oldPos->setPoints($oldPoints + $points);
					$oldCount = $oldPos->getTournamentCount();
					$oldPos->setTournamentCount($oldCount + 1);
					return;
				}
			}
		}
		$newPos = new Tx_DuRanking_Domain_Model_RankingPosition($participant, $points);
		$this->rankingPositionsFemale[] = $newPos;
	}
	
	/**
	 * Calculates the average
	 *
	 * @param Tx_DuRanking_Domain_Model_Season $season
	 * @return void
	 */
	protected  function calculateRankingPosition($season) {
		//number of tournaments and locations to qualify for final tournament - defined in season.
		$tourCount = $season->getTournamentCount();
		$locationCount = $season->getLocationCount();
		
		foreach($this->rankingPositions as $ranking) {
			//is qualified?
			if ($ranking->getTournamentCount() >= $tourCount) {
				$ranking->setIsQualified(True);
			}
			//calculate average (Points/tournamentCount) when qualified
			if ($ranking->isQualified()) {
				$average = $ranking->getPoints() / $ranking->getTournamentCount();
				$ranking->setAverage($average);
			}
		}
	}
	
	/**
	 * Calculates the average
	 *
	 * @param Tx_DuRanking_Domain_Model_Season $season
	 * @return void
	 */
	protected  function calculateRankingPositionMale($season) {
		//number of tournaments and locations to qualify for final tournament - defined in season.
		$tourCount = $season->getTournamentCount();
		$locationCount = $season->getLocationCount();
		
		foreach($this->rankingPositionsMale as $ranking) {
			//is qualified?
			if ($ranking->getTournamentCount() >= $tourCount) {
				$ranking->setIsQualified(True);
			}
			//calculate average (Points/tournamentCount) when qualified
			if ($ranking->isQualified()) {
				$average = $ranking->getPoints() / $ranking->getTournamentCount();
				$ranking->setAverage($average);
			}
		}
	}
	
	/**
	 * Calculates the average
	 *
	 * @param Tx_DuRanking_Domain_Model_Season $season
	 * @return void
	 */
	protected  function calculateRankingPositionFemale($season) {
		//number of tournaments and locations to qualify for final tournament - defined in season.
		$tourCount = $season->getTournamentCount();
		$locationCount = $season->getLocationCount();
		
		foreach($this->rankingPositionsFemale as $ranking) {
			//is qualified?
			if ($ranking->getTournamentCount() >= $tourCount) {
				$ranking->setIsQualified(True);
			}
			//calculate average (Points/tournamentCount) when qualified
			if ($ranking->isQualified()) {
				$average = $ranking->getPoints() / $ranking->getTournamentCount();
				$ranking->setAverage($average);
			}
		}
	}
	
	/**
	 * Sorting the ranking list
	 *
	 * @param int $sorting 	0 => points, 1 => average
	 * @return void
	 */
	protected  function sortRankingPosition($sorting) {
		
		if ($sorting == 0) {
			usort($this->rankingPositions, array("Tx_DuRanking_Domain_Model_RankingPosition", "comparePoints"));
		}
		elseif ($sorting == 1) {
			usort($this->rankingPositions, array("Tx_DuRanking_Domain_Model_RankingPosition", "compareAverage"));
		}
		
		$no = 1;
		foreach($this->rankingPositions as $ranking) {
			$ranking->setNumber($no);
			$no = $no + 1;
		}
	}
	
	/**
	 * Sorting the ranking list
	 *
	 * @param int $sorting 	0 => points, 1 => average
	 * @return void
	 */
	protected  function sortRankingPositionMale($sorting) {
		
		if ($sorting == 0) {
			usort($this->rankingPositionsMale, array("Tx_DuRanking_Domain_Model_RankingPosition", "comparePoints"));
		}
		elseif ($sorting == 1) {
			usort($this->rankingPositionsMale, array("Tx_DuRanking_Domain_Model_RankingPosition", "compareAverage"));
		}
		
		$no = 1;
		foreach($this->rankingPositionsMale as $ranking) {
			$ranking->setNumber($no);
			$no = $no + 1;
		}
	}
	
	/**
	 * Sorting the ranking list
	 *
	 * @param int $sorting 	0 => points, 1 => average
	 * @return void
	 */
	protected  function sortRankingPositionFemale($sorting) {
		
		if ($sorting == 0) {
			usort($this->rankingPositionsFemale, array("Tx_DuRanking_Domain_Model_RankingPosition", "comparePoints"));
		}
		elseif ($sorting == 1) {
			usort($this->rankingPositionsFemale, array("Tx_DuRanking_Domain_Model_RankingPosition", "compareAverage"));
		}
		
		$no = 1;
		foreach($this->rankingPositionsFemale as $ranking) {
			$ranking->setNumber($no);
			$no = $no + 1;
		}
	}
}
?>